/**
 ***********************************************************************
 * $Id: helper.c 326 2015-01-05 17:53:59Z mkwayisi $
 * ---------------------------------------------------------------------
 * Authored by Michael Kwayisi. Copyright (c) 2014. See license below.
 * Comments are appreciated - mic at kwayisi dot org | www.kwayisi.org
 * ---------------------------------------------------------------------
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are stringently met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions, and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions, and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. The end-user documentation included with the redistribution,
 *     if any, must include the following acknowledgment:
 *       "This product includes software written by Michael Kwayisi."
 *     Alternately, this acknowledgment may appear in the software
 *     itself, if and wherever such 3rd-party acknowledgments appear.
 *  4. Neither the name of the software nor the name of its author
 *     and/or contributors may be used to endorse or promote products
 *     derived from this software without specific prior permission.
 ***********************************************************************
 */

#include <windows.h>
#include "helper.h"

//==[ CODE ]============================================================
// CenterWindowToScreen
VOID CenterWindowToScreen(HWND hwnd)
{
	RECT rcScreen;
	RECT rcWindow;
	
	GetWindowRect(hwnd, &rcWindow);
	SystemParametersInfo(SPI_GETWORKAREA, 0, &rcScreen, 0);
	
	SetWindowPos(hwnd, NULL,
		(rcScreen.right - (rcWindow.right - rcWindow.left)) / 2,
		(rcScreen.bottom - (rcWindow.bottom - rcWindow.top)) / 2,
		0, 0, SWP_NOZORDER | SWP_NOSIZE);
}

//==[ CODE ]============================================================
// CenterWindowToParent
VOID CenterWindowToParent(HWND hwnd)
{
	RECT rc, rcp;
	GetWindowRect(hwnd, &rc);
	GetWindowRect(GetParent(hwnd), &rcp);
	SetWindowPos(hwnd, NULL,
		rcp.left + ((rcp.right - rcp.left) - (rc.right - rc.left)) / 2,
		rcp.top + ((rcp.bottom - rcp.top) - (rc.bottom - rc.top)) / 2,
		0, 0, SWP_NOZORDER | SWP_NOSIZE);
}

//==[ CODE ]============================================================
// GetWindowSize
BOOL GetWindowSize(HWND hwnd, LPSIZE size)
{
	RECT rc;
	if (GetWindowRect(hwnd, &rc)) {
		size->cx = rc.right - rc.left;
		size->cy = rc.bottom - rc.top;
		return TRUE;
	}
	return FALSE;
}

//==[ CODE ]============================================================
// SetWindowSize
BOOL SetWindowSize(HWND hwnd, INT nWidth, INT nHeight)
{
	return SetWindowPos(hwnd, NULL,
		0, 0, nWidth, nHeight, SWP_NOZORDER | SWP_NOMOVE);
}

//==[ CODE ]============================================================
// GetWindowClientSize
BOOL GetWindowClientSize(HWND hwnd, LPSIZE size)
{
	RECT rc;
	if (GetClientRect(hwnd, &rc)) {
		size->cx = rc.right;
		size->cy = rc.bottom;
		return TRUE;
	}
	return FALSE;
}

//==[ CODE ]============================================================
// SetWindowClientSize
BOOL SetWindowClientSize(HWND hwnd, INT width, INT height)
{
	SIZE fSize = { 0, 0 };
	SIZE cSize = { 0, 0 };
	GetWindowSize(hwnd, &fSize);
	GetWindowClientSize(hwnd, &cSize);
	fSize.cx += (width - cSize.cx);
	fSize.cy += (height - cSize.cy);
	return SetWindowPos(hwnd, NULL,
		0, 0, fSize.cx, fSize.cy, SWP_NOZORDER | SWP_NOMOVE);
}

//==[ CODE ]============================================================
// CreateSimpleFont
HFONT CreateSimpleFont(int nPointSize, LPCTSTR lpszFace)
{
	return CreateFont(
		-MulDiv(nPointSize, GetDeviceCaps(GetDC(NULL), LOGPIXELSY), 72),
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, lpszFace);
}

//==[ CODE ]============================================================
// SetChildWindowFont
BOOL CALLBACK SetChildWindowFont(HWND hwnd, LPARAM lParam)
{
	SendMessage(hwnd, WM_SETFONT, (WPARAM) lParam, TRUE);
	return TRUE;
}

//==[ CODE ]============================================================
// EnableChildWindow
BOOL CALLBACK EnableChildWindow(HWND hwnd, LPARAM lParam)
{
	EnableWindow(hwnd, (BOOL) lParam);
	return TRUE;
}
