/**
 ***********************************************************************
 * $Id: main.c 326 2015-01-05 17:53:59Z mkwayisi $
 * ---------------------------------------------------------------------
 * Authored by Michael Kwayisi. Copyright (c) 2014. See license below.
 * Comments are appreciated - mic at kwayisi dot org | www.kwayisi.org
 * ---------------------------------------------------------------------
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are stringently met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions, and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions, and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. The end-user documentation included with the redistribution,
 *     if any, must include the following acknowledgment:
 *       "This product includes software written by Michael Kwayisi."
 *     Alternately, this acknowledgment may appear in the software
 *     itself, if and wherever such 3rd-party acknowledgments appear.
 *  4. Neither the name of the software nor the name of its author
 *     and/or contributors may be used to endorse or promote products
 *     derived from this software without specific prior permission.
 ***********************************************************************
 */

#include <windows.h>
#include <commctrl.h>
#include <tchar.h>
#include <stdio.h>

#include "sqlite3.h"
#include "helper.h"
#include "common.h"
#include "main.h"
#include "design.h"

#define GETSTRING(x)	(LoadString(g_pActiveResource->hInstance, x, \
							g_szBuffer, COUNTOF(g_szBuffer)), g_szBuffer)

PRESOURCE *g_paResources = NULL;
PRESOURCE  g_pActiveResource = NULL;
USHORT g_nResourcesCount = 0;
TCHAR g_szBuffer[64];

HSQLITE g_hSQLite = NULL;
HANDLE g_hPlayTimeThread = NULL;

// Application byte data.
// LOBYTE of LOWORD = Start hour   | LOBYTE of HIWORD = Interval
// HIBYTE of LOWORD = Start minute | HIBYTE of HIWORD = RESERVED
DWORD g_dwAppData = 0;

// Application-specific internal flags.
DWORD g_dwAppFlags = 0;

//==[ CODE ]============================================================
// WinMain
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
	LPSTR lpCmdline, int nCmdShow)
{
	HANDLE hMutex;
	HWND hwnd;
	MSG msg;
	WNDCLASSEX wcex;
	int i;
	
	// Check if the application is already running.
	// If it is, bring it to the foreground so the user wouldn't bug us again.
	// We'll as well terminate this initiated process.
	hMutex = CreateMutex(NULL, TRUE, STR_MUTEX);
	if (GetLastError() == ERROR_ALREADY_EXISTS) {
		hwnd = FindWindow(STR_WNDCLASS, NULL);
		if (hwnd != NULL) {
			SendMessage(hwnd, WM_COMMAND, MAKEWPARAM(IDM_PREFS, BN_CLICKED), 0);
			SetForegroundWindow(hwnd);
		}
		return 0;
	}
	
	// SPOILER ALERT!!!
	// This is pretty silly! I'm using 'nCmdShow' to get additional info
	// from the 'Initialize' function since I'm NEVER going to use it here :)
	if(!Initialize(&nCmdShow)) {
		MessageBox(NULL, STR_INITERROR, STR_APPTITLE, MB_OK | MB_ICONERROR);
		return 1;
	}
	
	// If we're running for the first time, set the application defaults.
	// CAUTION: When the application is running for the first time g_dwAppData
	// will be zero which means the value of 'Interval' will also be zero.
	// This will make SchedNextTimer to run into an INFINITE LOOP when called!
	// How 'bout that for a software, huh? OK, let's fix that bug below.
	if (nCmdShow) {
		g_dwAppData = MAKELONG(MAKEWORD(0, 0), MAKEWORD(15, 0));
		g_dwAppFlags = APP_AUTOMATIC | APP_AUTOSTART;
		SaveSettings();
	}
	
	wcex.cbSize = sizeof(WNDCLASSEX);
	wcex.style = 0;
	wcex.lpszMenuName = NULL;
	wcex.lpszClassName = STR_WNDCLASS;
	wcex.cbWndExtra = 0;
	wcex.cbClsExtra = 0;
	wcex.hInstance = hInstance;
	wcex.lpfnWndProc = WndProc;
	wcex.hbrBackground = (HBRUSH) COLOR_3DSHADOW;
	wcex.hCursor = LoadCursor(NULL, IDC_ARROW);
	wcex.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_MAIN));
	wcex.hIconSm = (HICON) LoadImage(hInstance,
		MAKEINTRESOURCE(IDI_MAIN), IMAGE_ICON, 16, 16, 0);
	
	if (!RegisterClassEx(&wcex)) {
		MessageBox(NULL, STR_WNDCLASSERROR, STR_APPTITLE, MB_OK | MB_ICONERROR);
		return 1;
	}
	
	hwnd = CreateWindowEx(WS_EX_CONTROLPARENT, wcex.lpszClassName, NULL,
		WS_POPUP | WS_CAPTION | WS_SYSMENU,
		0, 0, 100, 100, NULL, NULL, hInstance, NULL);
	
	if (hwnd == NULL) {
		MessageBox(NULL, STR_HWNDERROR, STR_APPTITLE, MB_OK | MB_ICONERROR);
		return 1;
	}
	
	/**
	 * WARNING:
	 * THE APPICATION MUST NOT EXIT UNEXPECTEDLY BEYOND THIS POINT.
	 * IF IT NEEDS TO, THEN IT MUST SEND A WM_DESTROY MESSAGE TO THE MAIN
	 * WINDOW PROCEDURE SO THAT RESOURCES THAT HAVE BEEN ALLOCATED CAN BE
	 * PROPERLY DISPOSED BEFORE THE APPLICATION EXITS.
	 */
	EnumChildWindows(hwnd, SetChildWindowFont,
		(LPARAM) CreateSimpleFont(INT_FONT_SIZE, STR_FONT_FACE));
	SetControlsText(hwnd);
	
	// Language
	for (i = 0; i < g_nResourcesCount; i++) {
		SendDlgItemMessage(hwnd, IDC_CBX_LANGUAGE,
			CB_ADDSTRING, 0, (LPARAM) g_paResources[i]->pzLanguage);
		SendDlgItemMessage(hwnd, IDC_CBX_LANGUAGE,
			CB_SETITEMDATA, i, (LPARAM) g_paResources[i]);
	}
	i = SendDlgItemMessage(hwnd, IDC_CBX_LANGUAGE,
		CB_FINDSTRINGEXACT, -1, (LPARAM) g_pActiveResource->pzLanguage);
	if (i == CB_ERR) i = 0;
	SendDlgItemMessage(hwnd, IDC_CBX_LANGUAGE, CB_SETCURSEL, i, 0);
	
	// Start hour
	i = LOBYTE(LOWORD(g_dwAppData));
	SendDlgItemMessage(hwnd, IDC_EDT_STARTHOUR, EM_LIMITTEXT, 2, 0);
	SendDlgItemMessage(hwnd, IDC_UPD_STARTHOUR,
		UDM_SETRANGE, 0, (LPARAM) MAKELONG(12, 1));
	// The expression { i % 12 ? i % 12 : 12 } is meant to be like that.
	// I once substituted it with just { i ? i % 12 : 12 } which is very
	// different as when the time is 12 noon we should show it as such.
	SendDlgItemMessage(hwnd, IDC_UPD_STARTHOUR,
		UDM_SETPOS, 0, (LPARAM) MAKELONG(i % 12 ? i % 12 : 12, 0));
	SendDlgItemMessage(hwnd, IDC_CBX_STARTTIME,
		CB_SETCURSEL, i >= 12 ? 1 : 0, 0);
	
	// Start minute
	i = HIBYTE(LOWORD(g_dwAppData));
	SendDlgItemMessage(hwnd, IDC_EDT_STARTMINUTE, EM_LIMITTEXT, 2, 0);
	SendDlgItemMessage(hwnd, IDC_UPD_STARTMINUTE,
		UDM_SETRANGE, 0, (LPARAM) MAKELONG(59, 0));
	SetDlgItemInt(hwnd, IDC_EDT_STARTMINUTE, i, FALSE);
	SendMessage(hwnd, WM_COMMAND,	// Set in %02d format.
		MAKEWPARAM(IDC_EDT_STARTMINUTE, EN_KILLFOCUS),
			(LPARAM) GetDlgItem(hwnd, IDC_EDT_STARTMINUTE));
	
	// Interval
	i = LOBYTE(HIWORD(g_dwAppData));
	SendDlgItemMessage(hwnd, IDC_EDT_INTERVAL, EM_LIMITTEXT, 2, 0);
	SendDlgItemMessage(hwnd, IDC_UPD_INTERVAL, UDM_SETRANGE, 0, (LPARAM) MAKELONG(59, 1));
	SetDlgItemInt(hwnd, IDC_EDT_INTERVAL, i > 59 ? i - 59 : i, FALSE);
	SendDlgItemMessage(hwnd, IDC_CBX_INTERVAL, CB_SETCURSEL, i > 59 ? 1 : 0, 0);
	
	// Schedule
	SendDlgItemMessage(hwnd, IDC_LBX_SCHEDULE, LB_SETCOLUMNWIDTH,
		IDC_LBX_SCHEDULE_W / 2 - 2, 0);
	{
	HSQLITESTMT hSQLiteStmt;
	if (SQLitePrepare(g_hSQLite, TEXT("SELECT [Time] FROM [Schedule]"),
		sizeof(TCHAR) * 29, &hSQLiteStmt, NULL) == SQLITE_OK)
	{
		while (SQLiteStep(hSQLiteStmt) == SQLITE_ROW)
			ScheduleTime(hwnd, SQLiteColumnInt(hSQLiteStmt, 0), FALSE);
	}
	SQLiteFinalize(hSQLiteStmt);
	}
	
	// Miscellaneous
	SetWindowClientSize(hwnd, IDC_FRM_MAIN_W, IDC_FRM_MAIN_H);
	CenterWindowToScreen(hwnd);
	
	// We sending a command notification message to the auto/sched radio
	// buttons to set the initial state. Since this is app-triggered and
	// not from the user (knowing that the notification handler will flip
	// the APP_AUTOMATIC flag on/off) we need to flip it ahead.
	// So if it's TRUE, we say it's FALSE. If it's FALSE we say it's TRUE.
	// Not sure if that makes sense ;)
	g_dwAppFlags ^= APP_AUTOMATIC;
	SendDlgItemMessage(hwnd,
		(g_dwAppFlags & APP_AUTOMATIC) ? IDC_RBT_SCHEDULE : IDC_RBT_AUTOMATIC,
			BM_CLICK, 0, 0);
	
	// Set the AutoStart state.
	// We're using the same algorithm as the one for Auto/Shed above.
	g_dwAppFlags ^= APP_AUTOSTART;
	SendMessage(hwnd, WM_COMMAND, MAKEWPARAM(IDM_AUTOSTART, BN_CLICKED), 0);
	
	// SPOILER ALERT!!!
	// The additional info gotten from the initializing function has NOT been
	// modified since, so we can safely use it here. I especially delayed the
	// display of the welcome message so as to avoid any delay in between its
	// showing and that of the preferences window. Otherwise, a geek would say
	// "Why is a native app running so slow? The developer must be a bad one!"
	if (nCmdShow) {
		MessageBox(hwnd, STR_NEWINSTALL, STR_APPTITLE, MB_OK | MB_ICONINFORMATION);
	}
	
	// SPOILER ALERT!!!
	// Determine if we have to show the main window. If the process was
	// started at system startup (with any parameter), then the window must be
	// hidden until the user requests it to be shown. However if the user
	// explicitly launched the application (without parameter) then we must
	// immediately show the window.
	nCmdShow = *lpCmdline == 0;
	
	SetFocus(hwnd);
	SchedNextTimer(hwnd);
	
	if (nCmdShow) {
		ShowWindow(hwnd, SW_SHOWDEFAULT);
		UpdateWindow(hwnd);
	}
	
	while (GetMessage(&msg, NULL, 0, 0) > 0) {
		if (!IsDialogMessage(hwnd, &msg)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
	
	// Release the mutex. I'm not sure if this is even neccessary,
	// but it wouldn't hurt, right?
	ReleaseMutex(hMutex);
	
	return 0;
}

//==[ CODE ]============================================================
// WndProc
LRESULT CALLBACK WndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	static NOTIFYICONDATA nid;
	static HBITMAP hbmLogo = NULL;
	switch (uMsg) {
		case WM_CREATE:
			// Make the shell notify icon.
			nid.cbSize = sizeof(NOTIFYICONDATA);
			nid.hWnd = hwnd;
			nid.uID = IDC_NOTIFYICON;
			nid.uCallbackMessage = WM_NOTIFYICON;
			nid.hIcon = LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_MAIN));
			_tcscpy(nid.szTip, STR_APPFULLNAME);
			nid.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
			
			// If the creation of the icon fails, we need to terminate the
			// application since it serves as the control interface for the user.
			// TODO: As it is now, there's a flicker of the app window
			// just before it exits :(
			if (!Shell_NotifyIcon(NIM_ADD, &nid)) {
				MessageBox(hwnd, STR_ICONERROR, STR_APPTITLE, MB_OK | MB_ICONERROR);
				SendMessage(hwnd, WM_DESTROY, 0, 0);
			}
			
			hbmLogo =
				LoadBitmap(GetModuleHandle(NULL), MAKEINTRESOURCE(IDB_LOGO));
			
			CreateWindowEx(0, WC_STATIC, NULL,
				WS_CHILD | WS_VISIBLE | SS_CENTERIMAGE,
				IDC_LBL_LANGUAGE_X, IDC_LBL_LANGUAGE_Y,
				IDC_LBL_LANGUAGE_W, IDC_LBL_LANGUAGE_H,
				hwnd, (HMENU) IDC_LBL_LANGUAGE, GetModuleHandle(NULL), NULL);
			CreateWindowEx(0, WC_COMBOBOX, NULL,
				WS_CHILD | WS_VISIBLE | WS_TABSTOP | CBS_DROPDOWNLIST,
				IDC_CBX_LANGUAGE_X, IDC_CBX_LANGUAGE_Y,
				IDC_CBX_LANGUAGE_W, IDC_CBX_LANGUAGE_H,
				hwnd, (HMENU) IDC_CBX_LANGUAGE, GetModuleHandle(NULL), NULL);
			CreateWindowEx(0, WC_STATIC, NULL,
				WS_CHILD | WS_VISIBLE | SS_CENTERIMAGE,
				IDC_LBL_VOICE_X, IDC_LBL_VOICE_Y,
				IDC_LBL_VOICE_W, IDC_LBL_VOICE_H,
				hwnd, (HMENU) IDC_LBL_VOICE, GetModuleHandle(NULL), NULL);
			CreateWindowEx(0, WC_COMBOBOX, NULL,
				WS_CHILD | WS_VISIBLE | WS_TABSTOP | CBS_DROPDOWNLIST | CBS_SORT,
				IDC_CBX_VOICE_X, IDC_CBX_VOICE_Y,
				IDC_CBX_VOICE_W, IDC_CBX_VOICE_H,
				hwnd, (HMENU) IDC_CBX_VOICE, GetModuleHandle(NULL), NULL);
			
			CreateWindowEx(0, WC_BUTTON, NULL,
				WS_CHILD | WS_VISIBLE | BS_GROUPBOX,
				IDC_GBX_AUTOMATIC_X, IDC_GBX_AUTOMATIC_Y,
				IDC_GBX_AUTOMATIC_W, IDC_GBX_AUTOMATIC_H,
				hwnd, (HMENU) IDC_GBX_AUTOMATIC, GetModuleHandle(NULL), NULL);
			CreateWindowEx(0, WC_BUTTON, NULL,
				WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_RADIOBUTTON,
				IDC_RBT_AUTOMATIC_X, IDC_RBT_AUTOMATIC_Y, 0, 0,
				hwnd, (HMENU) IDC_RBT_AUTOMATIC, GetModuleHandle(NULL), NULL);
			CreateWindowEx(0, WC_STATIC, NULL,
				WS_CHILD | WS_VISIBLE | SS_CENTERIMAGE,
				IDC_LBL_STARTTIME_X, IDC_LBL_STARTTIME_Y,
				IDC_LBL_STARTTIME_W, IDC_LBL_STARTTIME_H,
				hwnd, (HMENU) IDC_LBL_STARTTIME, GetModuleHandle(NULL), NULL);
			CreateWindowEx(WS_EX_CLIENTEDGE, WC_EDIT, NULL,
				WS_CHILD | WS_VISIBLE | WS_TABSTOP | ES_NUMBER,
				IDC_EDT_STARTHOUR_X, IDC_EDT_STARTHOUR_Y,
				IDC_EDT_STARTHOUR_W, IDC_EDT_STARTHOUR_H,
				hwnd, (HMENU) IDC_EDT_STARTHOUR, GetModuleHandle(NULL), NULL);
			CreateWindowEx(0, UPDOWN_CLASS, NULL,
				WS_CHILD | WS_VISIBLE | UDS_AUTOBUDDY | UDS_SETBUDDYINT
					| UDS_ALIGNRIGHT | UDS_ARROWKEYS, 0, 0, 0, 0,
				hwnd, (HMENU) IDC_UPD_STARTHOUR, GetModuleHandle(NULL), NULL);
			CreateWindowEx(WS_EX_CLIENTEDGE, WC_EDIT, NULL,
				WS_CHILD | WS_VISIBLE | WS_TABSTOP | ES_NUMBER,
				IDC_EDT_STARTMINUTE_X, IDC_EDT_STARTMINUTE_Y,
				IDC_EDT_STARTMINUTE_W, IDC_EDT_STARTMINUTE_H,
				hwnd, (HMENU) IDC_EDT_STARTMINUTE, GetModuleHandle(NULL), NULL);
			CreateWindowEx(0, UPDOWN_CLASS, NULL,
				WS_CHILD | WS_VISIBLE | UDS_AUTOBUDDY | UDS_SETBUDDYINT
					| UDS_ALIGNRIGHT | UDS_ARROWKEYS, 0, 0, 0, 0,
				hwnd, (HMENU) IDC_UPD_STARTMINUTE, GetModuleHandle(NULL), NULL);
			CreateWindowEx(0, WC_COMBOBOX, NULL,
				WS_CHILD | WS_VISIBLE | WS_TABSTOP | CBS_DROPDOWNLIST,
				IDC_CBX_STARTTIME_X, IDC_CBX_STARTTIME_Y,
				IDC_CBX_STARTTIME_W, IDC_CBX_STARTTIME_H,
				hwnd, (HMENU) IDC_CBX_STARTTIME, GetModuleHandle(NULL), NULL);
			CreateWindowEx(0, WC_STATIC, NULL,
				WS_CHILD | WS_VISIBLE | SS_CENTERIMAGE,
				IDC_LBL_INTERVAL_X, IDC_LBL_INTERVAL_Y,
				IDC_LBL_INTERVAL_W, IDC_LBL_INTERVAL_H,
				hwnd, (HMENU) IDC_LBL_INTERVAL, GetModuleHandle(NULL), NULL);
			CreateWindowEx(WS_EX_CLIENTEDGE, WC_EDIT, NULL,
				WS_CHILD | WS_VISIBLE | WS_TABSTOP | ES_NUMBER,
				IDC_EDT_INTERVAL_X, IDC_EDT_INTERVAL_Y,
				IDC_EDT_INTERVAL_W, IDC_EDT_INTERVAL_H,
				hwnd, (HMENU) IDC_EDT_INTERVAL, GetModuleHandle(NULL), NULL);
			CreateWindowEx(0, UPDOWN_CLASS, NULL,
				WS_CHILD | WS_VISIBLE | UDS_AUTOBUDDY | UDS_SETBUDDYINT
					| UDS_ALIGNRIGHT | UDS_ARROWKEYS, 0, 0, 0, 0,
				hwnd, (HMENU) IDC_UPD_INTERVAL, GetModuleHandle(NULL), NULL);
			CreateWindowEx(0, WC_COMBOBOX, NULL,
				WS_CHILD | WS_VISIBLE | WS_TABSTOP | CBS_DROPDOWNLIST,
				IDC_CBX_INTERVAL_X, IDC_CBX_INTERVAL_Y,
				IDC_CBX_INTERVAL_W, IDC_CBX_INTERVAL_H,
				hwnd, (HMENU) IDC_CBX_INTERVAL, GetModuleHandle(NULL), NULL);
			
			CreateWindowEx(0, WC_BUTTON, NULL,
				WS_CHILD | WS_VISIBLE | BS_GROUPBOX,
				IDC_GBX_SCHEDULE_X, IDC_GBX_SCHEDULE_Y,
				IDC_GBX_SCHEDULE_W, IDC_GBX_SCHEDULE_H,
				hwnd, (HMENU) IDC_GBX_SCHEDULE, GetModuleHandle(NULL), NULL);
			CreateWindowEx(0, WC_BUTTON, NULL,
				WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_RADIOBUTTON,
				IDC_RBT_SCHEDULE_X, IDC_RBT_SCHEDULE_Y, 0, 0,
				hwnd, (HMENU) IDC_RBT_SCHEDULE, GetModuleHandle(NULL), NULL);
			CreateWindowEx(WS_EX_CLIENTEDGE, WC_LISTBOX, NULL,
				WS_CHILD | WS_VISIBLE | WS_TABSTOP | WS_HSCROLL
					| LBS_NOINTEGRALHEIGHT | LBS_MULTICOLUMN
					| LBS_DISABLENOSCROLL | LBS_EXTENDEDSEL | LBS_NOTIFY
					| LBS_WANTKEYBOARDINPUT,
				IDC_LBX_SCHEDULE_X, IDC_LBX_SCHEDULE_Y,
				IDC_LBX_SCHEDULE_W, IDC_LBX_SCHEDULE_H,
				hwnd, (HMENU) IDC_LBX_SCHEDULE, GetModuleHandle(NULL), NULL);
			CreateWindowEx(0, WC_BUTTON, NULL,
				WS_CHILD | WS_VISIBLE | WS_TABSTOP,
				IDC_BTN_ADD_X, IDC_BTN_ADD_Y,
				IDC_BTN_ADD_W, IDC_BTN_ADD_H,
				hwnd, (HMENU) IDC_BTN_ADD, GetModuleHandle(NULL), NULL);
			CreateWindowEx(0, WC_BUTTON, NULL,
				WS_CHILD | WS_VISIBLE | WS_TABSTOP | WS_DISABLED,
				IDC_BTN_REMOVE_X, IDC_BTN_REMOVE_Y,
				IDC_BTN_REMOVE_W, IDC_BTN_REMOVE_H,
				hwnd, (HMENU) IDC_BTN_REMOVE, GetModuleHandle(NULL), NULL);
			
			CreateWindowEx(0, WC_BUTTON, NULL,
				WS_CHILD | WS_VISIBLE | WS_TABSTOP,
				IDC_BTN_TELLTIME_X, IDC_BTN_TELLTIME_Y,
				IDC_BTN_TELLTIME_W, IDC_BTN_TELLTIME_H,
				hwnd, (HMENU) IDC_BTN_TELLTIME, GetModuleHandle(NULL), NULL);
			CreateWindowEx(0, WC_BUTTON, NULL,
				WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_DEFPUSHBUTTON,
				IDC_BTN_SAVE_X, IDC_BTN_SAVE_Y,
				IDC_BTN_SAVE_W, IDC_BTN_SAVE_H,
				hwnd, (HMENU) IDC_BTN_SAVE, GetModuleHandle(NULL), NULL);
			CreateWindowEx(0, WC_BUTTON, NULL,
				WS_CHILD | WS_VISIBLE | WS_TABSTOP,
				IDC_BTN_CLOSE_X, IDC_BTN_CLOSE_Y,
				IDC_BTN_CLOSE_W, IDC_BTN_CLOSE_H,
				hwnd, (HMENU) IDC_BTN_CLOSE, GetModuleHandle(NULL), NULL);
			break;
		
		case WM_PAINT: {
			PAINTSTRUCT ps;
			HBITMAP hbmOld;
			BITMAP bm;
			HDC hdc;
			HDC hdcMem;
			
			hdc = BeginPaint(hwnd, &ps);
			hdcMem = CreateCompatibleDC(hdc);
			hbmOld = SelectObject(hdcMem, hbmLogo);
			
			GetObject(hbmLogo, sizeof(bm), &bm);
			BitBlt(hdc, 0, 0, bm.bmWidth, bm.bmHeight, hdcMem, 0, 0, SRCCOPY);
			
			SelectObject(hdcMem, hbmOld);
			DeleteDC(hdcMem);
			EndPaint(hwnd, &ps);
		}	break;
		
		case WM_COMMAND:
			switch (HIWORD(wParam)) {
				case BN_CLICKED:
					switch (LOWORD(wParam)) {
						case IDM_PREFS:
							if (!IsWindowVisible(hwnd)) {
								CenterWindowToScreen(hwnd);
								ShowWindow(hwnd, SW_SHOW);
							}
							SetForegroundWindow(hwnd);
							break;
						
						case IDM_AUTOSTART: {
							HKEY hKey;
							TCHAR szPath[MAX_PATH];
							
							GetModuleFileName(GetModuleHandle(NULL),
								szPath, COUNTOF(szPath));
							_sntprintf(szPath, COUNTOF(szPath),
								TEXT("%s %s"), szPath, TEXT("-z"));
							
							if (RegCreateKeyEx(HKEY_CURRENT_USER,
								STR_REGSUBKEY, 0, NULL, REG_OPTION_NON_VOLATILE,
									KEY_ALL_ACCESS, NULL, &hKey, NULL)
								!= ERROR_SUCCESS)
							{ break; }
							
							if (g_dwAppFlags & APP_AUTOSTART) {
								RegDeleteValue(hKey, STR_APPNAME);
							} else {
								RegSetValueEx(hKey, STR_APPNAME, 0, REG_SZ,
									(CONST BYTE *) szPath, sizeof(szPath));
							}
							
							RegCloseKey(hKey);
							g_dwAppFlags ^= APP_AUTOSTART;
							SaveSettings();
						}	break;
						
						case IDM_SILENT:
							g_dwAppFlags ^= APP_SILENT;
							SaveSettings();
							break;
						
						case IDM_EXIT:
							if (MessageBox(hwnd, GETSTRING(IDS_CONFIRMEXIT),
								STR_APPTITLE, MB_YESNOCANCEL | MB_DEFBUTTON2
									| MB_ICONWARNING) == IDYES)
							{
								PostMessage(hwnd, WM_DESTROY, 0, 0);
							}
							break;
						
						case IDC_RBT_AUTOMATIC:
						case IDC_RBT_SCHEDULE: {
							// The array is a structure for the two groups of
							// controls that we have. The radio button begins
							// the group and the rest of the members follow.
							// A group MUST be delimited with a zero.
							int ctls[] = {
								IDC_RBT_AUTOMATIC,
								IDC_LBL_STARTTIME, IDC_EDT_STARTHOUR, IDC_UPD_STARTHOUR,
								IDC_EDT_STARTMINUTE, IDC_UPD_STARTMINUTE, IDC_CBX_STARTTIME,
								IDC_LBL_INTERVAL, IDC_EDT_INTERVAL, IDC_UPD_INTERVAL,
								IDC_CBX_INTERVAL,
								0,
								IDC_RBT_SCHEDULE,
								IDC_LBX_SCHEDULE, IDC_BTN_ADD,
								0
							};
							
							// i = index of the start of the victorious group.
							// j = index of the start of the defeated group.
							// INFO: a victorious group is the group of
							// controls to enable while the defeated group is
							// the opposite.
							int i, j;
							j = LOWORD(wParam) == IDC_RBT_AUTOMATIC ?
								(i = 0, 12) : (i = 12, 0);
							
							if (!IsDlgButtonChecked(hwnd, ctls[i])) {
								// Set the check state flag.
								// It will set and unset as required.
								g_dwAppFlags ^= APP_AUTOMATIC;
								
								// Uncheck and disable the defeated controls
								// before checking and enabling the victors.
								CheckDlgButton(hwnd, ctls[j], BST_UNCHECKED);
								SetWindowLong(GetDlgItem(hwnd, ctls[j]), GWL_STYLE,
									WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_RADIOBUTTON);
								
								// At this point 'j' is no longer needed to
								// track of the defeated group so it is used
								// as a counter.
								for (j++; ctls[j]; j++)
									EnableWindow(GetDlgItem(hwnd, ctls[j]), FALSE);
								for (j = i + 1; ctls[j]; j++)
									EnableWindow(GetDlgItem(hwnd, ctls[j]), TRUE);
								
								SendMessage(hwnd, WM_COMMAND,
									MAKELONG(IDC_LBX_SCHEDULE, LBN_SELCHANGE),
										(LPARAM) GetDlgItem(hwnd, IDC_LBX_SCHEDULE));
								
								CheckDlgButton(hwnd, ctls[i], BST_CHECKED);
							}
						}	break;
						
						case IDC_BTN_ADD: {
							int nResult = DialogBox(GetModuleHandle(NULL),
								MAKEINTRESOURCE(IDD_SCHEDULE), hwnd, SchedDlgProc);
							if (nResult == IDC_BTN_SAVE) {
								// Refresh list box.
							}
							SetFocus((HWND) lParam);
						}	break;
						
						case IDC_BTN_REMOVE: {
							HSQLITESTMT hSQLiteStmt;
							LPINT lpnItems;
							int i;
							
							// I'm going blind this time. I've been coding
							// for far too long. Get a life, errors!
							i = SendDlgItemMessage(hwnd,
								IDC_LBX_SCHEDULE, LB_GETSELCOUNT, 0, 0);
							lpnItems = HeapAlloc(GetProcessHeap(),
								HEAP_ZERO_MEMORY, sizeof(INT) * i);
							if (lpnItems == NULL)
								break;
							
							SendDlgItemMessage(hwnd, IDC_LBX_SCHEDULE,
								LB_GETSELITEMS, i, (LPARAM) lpnItems);
							
							SQLitePrepare(g_hSQLite,
								TEXT("DELETE FROM [Schedule] WHERE [Time] = ?"),
									sizeof(TCHAR) * 39, &hSQLiteStmt, NULL);
							
							while (--i >= 0) {
								SQLiteBindInt(hSQLiteStmt, 1,
									SendDlgItemMessage(hwnd, IDC_LBX_SCHEDULE,
										LB_GETITEMDATA, lpnItems[i], 0));
								SQLiteStep(hSQLiteStmt);
								SQLiteReset(hSQLiteStmt);
								SendDlgItemMessage(hwnd, IDC_LBX_SCHEDULE,
									LB_DELETESTRING, lpnItems[i], 0);
							}
							
							SQLiteFinalize(hSQLiteStmt);
							HeapFree(GetProcessHeap(), 0, lpnItems);
							
							i = SendDlgItemMessage(hwnd,
								IDC_LBX_SCHEDULE, LB_GETCOUNT, 0, 0);
							if (!i) EnableWindow((HWND) lParam, FALSE);
						}	break;
						
						case IDC_BTN_TELLTIME:
							PostMessage(hwnd, WM_TIMER, 0, 0);
							break;
						
						case IDC_BTN_SAVE: {
							BYTE bHour, bMinute, bInterval;
							
							bHour = GetDlgItemInt(hwnd,
								IDC_EDT_STARTHOUR, NULL, FALSE) % 12;
							
							// If the meridian is PM, add 12 to the value of
							// the hour so we end up with a "24-hour" hour.
							if (SendDlgItemMessage(hwnd,
								IDC_CBX_STARTTIME, CB_GETCURSEL, 0, 0))
							{ bHour += 12; }
							
							bMinute = GetDlgItemInt(hwnd,
								IDC_EDT_STARTMINUTE, NULL, FALSE);
							
							bInterval = GetDlgItemInt(hwnd,
								IDC_EDT_INTERVAL, NULL, FALSE);
							
							// If the unit of the interval is in hours, add
							// 59 to the value gotten. The idea is that, if
							// the value is more than 59 then the value - 59
							// is the value in hours.
							if (SendDlgItemMessage(hwnd,
								IDC_CBX_INTERVAL, CB_GETCURSEL, 0, 0))
							{ bInterval += 59; }
							
							g_dwAppData =
								MAKELONG(MAKEWORD(bHour, bMinute),
									MAKEWORD(bInterval, 0));
							
							if (SaveSettings()) {
								SchedNextTimer(hwnd);
								MessageBox(hwnd, GETSTRING(IDS_PREFSAVED),
									STR_APPTITLE, MB_OK | MB_ICONINFORMATION);
							} else {
								MessageBox(hwnd, GETSTRING(IDS_PREFNOTSAVED),
									STR_APPTITLE, MB_OK | MB_ICONWARNING);
							}
							SetFocus((HWND) lParam);
						}	break;
						
						case IDC_BTN_CLOSE:
							SendMessage(hwnd, WM_CLOSE, 0, 0);
							break;
					}
					break;
				
				case CBN_SELCHANGE:
				//case LBN_SELCHANGE: // Same as above
					switch (LOWORD(wParam)) {
						case IDC_CBX_LANGUAGE: {
							DWORD dwData;
							dwData = SendMessage((HWND) lParam, CB_GETCURSEL, 0, 0);
							if (dwData != CB_ERR) {
								dwData = SendMessage((HWND) lParam, CB_GETITEMDATA, dwData, 0);
								if (dwData != CB_ERR) {
									g_pActiveResource = (PRESOURCE) dwData;
									SetControlsText(hwnd);
								}
							}
						}	break;
						
						case IDC_CBX_VOICE: {
							int i;
							GetWindowText((HWND) lParam, g_szBuffer, COUNTOF(g_szBuffer));
							
							for (i = 0; i < g_pActiveResource->nVoicesCount; i++) {
								if (!_tcscmp(g_pActiveResource->paVoices[i], g_szBuffer)) {
									g_pActiveResource->pzActiveVoice =
										g_pActiveResource->paVoices[i];
									break;
								}
							}
						}	break;
						
						case IDC_CBX_INTERVAL: {
							DWORD dwIndex, dwInterval;
							dwIndex = SendMessage((HWND) lParam, CB_GETCURSEL, 0, 0);
							SendDlgItemMessage(hwnd, IDC_EDT_INTERVAL,
								WM_GETTEXT, COUNTOF(g_szBuffer), (LPARAM) g_szBuffer);
							dwInterval = _ttoi(g_szBuffer);
							
							if (dwIndex == 1 && dwInterval > 24) {
								_sntprintf(g_szBuffer, COUNTOF(g_szBuffer),
									TEXT("%d"), 24);
								SendDlgItemMessage(hwnd, IDC_EDT_INTERVAL,
									WM_SETTEXT, 0, (LPARAM) g_szBuffer);
							}
						}	break;
						
						case IDC_LBX_SCHEDULE: {
							int i;
							i = SendMessage((HWND) lParam, LB_GETSELCOUNT, 0, 0);
							EnableWindow(GetDlgItem(hwnd, IDC_BTN_REMOVE),
								IsWindowEnabled((HWND) lParam) && i != LB_ERR && i > 0);
						}	break;
					}
					break;
				
				case EN_KILLFOCUS:
					switch (LOWORD(wParam)) {
						case IDC_EDT_STARTHOUR: {
							BYTE bHour;
							GetWindowText((HWND) lParam,
								g_szBuffer, COUNTOF(g_szBuffer));
							bHour = _ttoi(g_szBuffer);
							
							if (bHour == 0)
								bHour = 1;
							else if (bHour > 12)
								bHour = 12;
							else break;
							
							_sntprintf(g_szBuffer, COUNTOF(g_szBuffer),
								TEXT("%d"), bHour);
							SetWindowText((HWND) lParam, g_szBuffer);
						}	break;
						
						case IDC_EDT_STARTMINUTE: {
							BYTE bMinute;
							GetWindowText((HWND) lParam,
								g_szBuffer, COUNTOF(g_szBuffer));
							bMinute = _ttoi(g_szBuffer);
							
							if (bMinute > 9 && bMinute < 60)
								break;
							if (bMinute > 59)
								bMinute = 59;
							
							_sntprintf(g_szBuffer,
								COUNTOF(g_szBuffer), TEXT("%02d"), bMinute);
							SetWindowText((HWND) lParam, g_szBuffer);
						}	break;
						
						case IDC_EDT_INTERVAL: {
							DWORD dwInterval, dwIndex;
							dwIndex = SendDlgItemMessage(hwnd,
								IDC_CBX_INTERVAL, CB_GETCURSEL, 0, 0);
							SendMessage((HWND) lParam, WM_GETTEXT,
								COUNTOF(g_szBuffer), (LPARAM) g_szBuffer);
							
							dwInterval = _ttoi(g_szBuffer);
							if (dwInterval == 0)
								dwInterval = 1;
							else if (dwIndex == 0 && dwInterval > 59)
								dwInterval = 59;
							else if (dwIndex == 1 && dwInterval > 24)
								dwInterval = 24;
							else break;
							
							_sntprintf(g_szBuffer, COUNTOF(g_szBuffer),
								TEXT("%d"), dwInterval);
							SendMessage((HWND) lParam,
								WM_SETTEXT, 0, (LPARAM) g_szBuffer);
						}	break;
					}
					break;
			}
			break;
		
		case WM_VKEYTOITEM:
			switch (LOWORD(wParam)) {
				case VK_INSERT:
					SendMessage(hwnd, WM_COMMAND,
						MAKEWPARAM(IDC_BTN_ADD, BN_CLICKED),
							(LPARAM) GetDlgItem(hwnd, IDC_BTN_ADD));
					break;
				
				case VK_DELETE:
					SendMessage(hwnd, WM_COMMAND,
						MAKEWPARAM(IDC_BTN_REMOVE, BN_CLICKED),
							(LPARAM) GetDlgItem(hwnd, IDC_BTN_REMOVE));
					break;
			}
			return -1;
			break;
		
		case WM_NOTIFYICON:
			switch (lParam) {
				case WM_LBUTTONDBLCLK:
					SendMessage(hwnd, WM_COMMAND,
						MAKEWPARAM(IDM_PREFS, BN_CLICKED), 0);
					break;
					
				//case WM_LBUTTONUP:
				case WM_RBUTTONUP: {
					HMENU hMenu;
					MENUITEMINFO mii;
					POINT pt;
					int mdata[] = { IDM_PREFS, IDS_PREFS, IDM_AUTOSTART,
						IDS_AUTOSTART, IDM_SILENT, IDS_SILENT, IDM_EXIT, IDS_EXIT };
					
					hMenu = LoadMenu(GetModuleHandle(NULL), MAKEINTRESOURCE(IDR_NOTIFYMENU));
						
					memset(&mii, 0, sizeof(MENUITEMINFO));
					mii.cbSize = sizeof(MENUITEMINFO);
					mii.fMask = MIIM_TYPE;
					mii.fType = MFT_STRING;
					
					// SPOILER ALERT: Use of pt.x as counter.
					for (pt.x = 0; pt.x < COUNTOF(mdata); pt.x += 2) {
						GETSTRING(mdata[pt.x+1]);
						mii.dwTypeData = g_szBuffer;
						SetMenuItemInfo(hMenu, mdata[pt.x], FALSE, &mii);
					}
					
					SetMenuDefaultItem(GetSubMenu(hMenu, 0), IDM_PREFS, FALSE);
					
					mdata[0] = IDM_AUTOSTART;
					mdata[1] = APP_AUTOSTART;
					mdata[2] = IDM_SILENT;
					mdata[3] = APP_SILENT;
					
					for (pt.x = 0; pt.x < 4; pt.x += 2) {
						CheckMenuItem(hMenu, mdata[pt.x],
							(g_dwAppFlags & mdata[pt.x+1]) ? MF_CHECKED : MF_UNCHECKED);
					}
					
					GetCursorPos(&pt);
					
					// According to MSDN the application window must be
					// the foreground window prior to tracking the popup menu
					// otherwise the context menu will not disappear should
					// the user click outside the menu.
					SetForegroundWindow(hwnd);
					
					TrackPopupMenu(GetSubMenu(hMenu, 0), TPM_CENTERALIGN, pt.x, pt.y, 0, hwnd, NULL);
					DestroyMenu(hMenu);
				}	break;
			}
			break;
		
		case WM_TIMER: {
			DWORD dwExitCode;
			GetExitCodeThread(g_hPlayTimeThread, &dwExitCode);
			
			if (dwExitCode != STILL_ACTIVE) {
				if (g_pActiveResource->pfPlayTimeProc) {
					// Dont play if we're in silent mode.
					// But if the user triggered the action, then play.
					if (!wParam || !(g_dwAppFlags & APP_SILENT))
						g_hPlayTimeThread = CreateThread(NULL, 0,
							g_pActiveResource->pfPlayTimeProc, NULL, 0, NULL);
				} else {
					MessageBox(NULL, TEXT("Missing func"),
						STR_APPTITLE, MB_OK | MB_ICONWARNING);
				}
				
				// If this is system-triggered,
				// then schedule the next timer.
				if (wParam)
					SchedNextTimer(hwnd);
			}
		}	break;
		
		case WM_TIMECHANGE:
			SchedNextTimer(hwnd);
			break;
		
		case WM_CLOSE:
			ShowWindow(hwnd, SW_HIDE);
			//DestroyWindow(hwnd);
			break;
		
		case WM_DESTROY:
			Shell_NotifyIcon(NIM_DELETE, &nid);
			DeleteObject(hbmLogo);
			PostQuitMessage(0);
			break;
		
		default:
			return DefWindowProc(hwnd, uMsg, wParam, lParam);
	}
	
	return 0;
}

//==[ CODE ]============================================================
// Initialize
BOOL Initialize(LPINT lpNewInstall)
{
	INITCOMMONCONTROLSEX iccex;
	static PTCHAR aVoices[] = { TEXT("Robot") };
	TCHAR szPath[MAX_PATH] = { 0 };
	DWORD dwGeneric = 0;
	HRSRC hResInfo = NULL;
	HGLOBAL hResData = NULL;
	char *pResBytes = NULL;
	DWORD dwResSize = 0;
	HANDLE hFile = NULL;
	int i;
	
	*lpNewInstall = FALSE;
	
	// Get the stylish XP+ theme, YAY!
	iccex.dwSize = sizeof(INITCOMMONCONTROLSEX);
	iccex.dwICC = ICC_WIN95_CLASSES;
	InitCommonControlsEx(&iccex);
	
	dwGeneric = GetEnvironmentVariable(TEXT("APPDATA"), szPath, COUNTOF(szPath));
	
	// We can't continue if any of these conditions is not met:
	// 1. If the environment variable was not found, dwCount will be 0.
	// 2. If the buffer size is not enough, the function will return the size
	//    required which will therefore be more than what we passed to it
	//    minus one (the NULL terminator).
	// 3. The length of the concatenated string folders should not exceed the
	//    the size of the buffer.
	// NOTE: The total length of the path separator required for the path plus
	// the NUL byte is 4.
	if (dwGeneric == 0 || dwGeneric >= COUNTOF(szPath)
		|| dwGeneric + COUNTOF(STR_APPCOMPANY) + COUNTOF(STR_APPNAME)
		+ COUNTOF(STR_SQLITEDB) + 4 > COUNTOF(szPath))
	{
		return FALSE;
	}
	
	// Create the company and application folders (if not exists).
	for (i = 0; i < 2; i++) {
		_sntprintf(szPath, COUNTOF(szPath), TEXT("%s\\%s"), szPath,
			i == 0 ? STR_APPCOMPANY : STR_APPNAME);
		dwGeneric = GetFileAttributes(szPath);
		
		// Just in case the path refers to a file, delete it.
		if (dwGeneric != INVALID_FILE_ATTRIBUTES
			&& !(dwGeneric & FILE_ATTRIBUTE_DIRECTORY))
		{
			dwGeneric = INVALID_FILE_ATTRIBUTES;
			if (!DeleteFile(szPath))
				return FALSE;
		}
		
		if (dwGeneric == INVALID_FILE_ATTRIBUTES
			&& !CreateDirectory(szPath, NULL))
		{
			return FALSE;
		}
	}
	
	_sntprintf(szPath, COUNTOF(szPath), TEXT("%s\\%s"), szPath, STR_SQLITEDB);
	dwGeneric = GetFileAttributes(szPath);
	
	// If the SQLite DB file mysteriously turns out to be a directory..
	if (dwGeneric != INVALID_FILE_ATTRIBUTES
		&& dwGeneric & FILE_ATTRIBUTE_DIRECTORY)
	{
		dwGeneric = INVALID_FILE_ATTRIBUTES;
		if (!RemoveDirectory(szPath))
			return FALSE;
	}
	
	// Create the DB file if not exists.
	if (dwGeneric == INVALID_FILE_ATTRIBUTES) {
		hResInfo = FindResource(NULL, MAKEINTRESOURCE(IDR_SQLITEDB), RT_RCDATA);
		if (hResInfo == NULL)
			return FALSE;
		
		hResData = LoadResource(NULL, hResInfo);
		if (hResData == NULL)
			return FALSE;
		
		pResBytes = LockResource(hResData);
		if (pResBytes == NULL)
			return FALSE;
		
		dwResSize = SizeofResource(GetModuleHandle(NULL), hResInfo);
		if (dwResSize == 0)
			return FALSE;
		
		hFile = CreateFile(szPath, GENERIC_WRITE, 0,
			NULL, CREATE_NEW, FILE_ATTRIBUTE_NORMAL, NULL);
		if (hFile == INVALID_HANDLE_VALUE)
			return FALSE;
		
		i = WriteFile(hFile, pResBytes, dwResSize, &dwGeneric, NULL);
		CloseHandle(hFile);
		
		if (!i)
			return FALSE;
		*lpNewInstall = TRUE;
	}
	
	// Allocate memory for the resources.
	g_paResources =
		HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(PRESOURCE));
	if (g_paResources == NULL)
		return FALSE;
	
	// Make the default (embedded) resource.
	g_paResources[0] =
		HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(RESOURCE));
	if (g_paResources[0] == NULL) {
		HeapFree(GetProcessHeap(), 0, g_paResources);
		g_paResources = NULL;
		return FALSE;
	}
	
	g_paResources[0]->hInstance = GetModuleHandle(NULL);
	g_paResources[0]->pzLanguage = TEXT("English (Default)");
	g_paResources[0]->paVoices = aVoices;
	g_paResources[0]->nVoicesCount = 1;
	g_paResources[0]->pfPlayTimeProc = PlayTime;
	g_nResourcesCount = 1;
	
	// Load other resource DLLs.
	LoadResourceDLLs();
	
	// Open a new SQLite connection.
	i = SQLiteOpen(szPath, &g_hSQLite);
	if (i != SQLITE_OK) {
		SQLiteClose(g_hSQLite);
		return FALSE;
	}
	
	// Load user settings
	if (!LoadSettings())
		return FALSE;
	
	// There'll always be a default resource.
	if (g_pActiveResource == NULL)
		g_pActiveResource = g_paResources[0];
	
	return TRUE;
}

//==[ CODE ]============================================================
// SchedDlgProc
BOOL CALLBACK SchedDlgProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {
		case WM_INITDIALOG: {
			int ctls[] = { IDC_LBL_HOUR, IDC_LBL_MINUTE, IDC_LBL_MERIDIAN,
				IDC_BTN_SAVE, IDC_BTN_CANCEL };
			int i;
			
			SetWindowText(hwnd, GETSTRING(IDS_ADDSCHEDULE));
			
			for (i = 0; i < COUNTOF(ctls); i++)
				SetDlgItemText(hwnd, ctls[i], GETSTRING(ctls[i]));
			
			SendDlgItemMessage(hwnd, IDC_EDT_HOUR, EM_SETLIMITTEXT, 2, 0);
			SendDlgItemMessage(hwnd, IDC_EDT_MINUTE, EM_SETLIMITTEXT, 2, 0);
			
			SendDlgItemMessage(hwnd, IDC_UPD_HOUR, UDM_SETRANGE, 0, MAKELPARAM(12, 1));
			SendDlgItemMessage(hwnd, IDC_UPD_MINUTE, UDM_SETRANGE, 0, MAKELPARAM(59, 0));
			SetDlgItemInt(hwnd, IDC_EDT_HOUR, 1, FALSE);
			SetDlgItemText(hwnd, IDC_EDT_MINUTE, TEXT("00"));
			
			SendDlgItemMessage(hwnd, IDC_CBX_MERIDIAN,
				CB_ADDSTRING, 0, (LPARAM) GETSTRING(IDS_AM));
			SendDlgItemMessage(hwnd, IDC_CBX_MERIDIAN,
				CB_ADDSTRING, 0, (LPARAM) GETSTRING(IDS_PM));
			SendDlgItemMessage(hwnd, IDC_CBX_MERIDIAN, CB_SETCURSEL, 0, 0);
			
			CenterWindowToParent(hwnd);
		}	break;
			
		case WM_COMMAND:
			switch (HIWORD(wParam)) {
				case BN_CLICKED:
					if (LOWORD(wParam) == IDC_BTN_SAVE) {
						HSQLITESTMT hSQLiteStmt;
						WORD wHour, wMinute;
						DWORD dwTime;
						BOOL bSuccess = FALSE;
						int nResult;
						
						// Make sure the user doesn't catch us off-guard.
						SetFocus(hwnd);
						
						wHour = GetDlgItemInt(hwnd, IDC_EDT_HOUR, NULL, FALSE) % 12;
						wMinute = GetDlgItemInt(hwnd, IDC_EDT_MINUTE, NULL, FALSE);
						if (SendDlgItemMessage(hwnd, IDC_CBX_MERIDIAN, CB_GETCURSEL, 0, 0))
							wHour += 12;
						dwTime = MAKELONG(wMinute, wHour);
						
						if (SQLitePrepare(g_hSQLite,
							TEXT("SELECT [Time] FROM [Schedule] WHERE [Time] = ?"),
								sizeof(TCHAR) * 46, &hSQLiteStmt, NULL)
							!= SQLITE_OK)
						{ goto end; }
						
						SQLiteBindInt(hSQLiteStmt, 1, dwTime);
						nResult = SQLiteStep(hSQLiteStmt);
						SQLiteFinalize(hSQLiteStmt);
						
						if (nResult == SQLITE_ROW) {
							MessageBox(hwnd, GETSTRING(IDS_SCHEDEXISTS),
								STR_APPTITLE, MB_OK | MB_ICONWARNING);
							break;
						}
						
						nResult = SQLitePrepare(g_hSQLite,
							TEXT("INSERT INTO [Schedule] VALUES (?)"),
								sizeof(TCHAR) * 33, &hSQLiteStmt, NULL);
						if (nResult != SQLITE_OK)
							goto end;
						
						SQLiteBindInt(hSQLiteStmt, 1, dwTime);
						nResult = SQLiteStep(hSQLiteStmt);
						if (nResult != SQLITE_DONE)
							goto end;
						
						bSuccess = TRUE;
						
						end:
						SQLiteFinalize(hSQLiteStmt);
						
						if (bSuccess) {
							ScheduleTime(GetParent(hwnd), dwTime, TRUE);
						} else {
							MessageBox(hwnd, GETSTRING(IDS_SCHEDNOTSAVED),
								STR_APPTITLE, MB_OK | MB_ICONWARNING);
							break;
						}
					}
					EndDialog(hwnd, LOWORD(wParam));
					break;
				
				case EN_KILLFOCUS: {
					UINT uVal;
					uVal = GetDlgItemInt(hwnd, LOWORD(wParam), NULL, FALSE);
					
					switch (LOWORD(wParam)) {
						case IDC_EDT_HOUR:
							if (uVal == 0)
								uVal = 1;
							else if (uVal > 12)
								uVal = 12;
							else break;
							SetDlgItemInt(hwnd, LOWORD(wParam), uVal, FALSE);
							break;
						
						case IDC_EDT_MINUTE:
							if (uVal > 59)
								uVal = 59;
							else if (uVal > 9 && uVal < 60)
								break;
							_sntprintf(g_szBuffer, COUNTOF(g_szBuffer),
								TEXT("%02d"), uVal);
							SetDlgItemText(hwnd, IDC_EDT_MINUTE, g_szBuffer);
							break;
					}
				}	break;
			}
			break;
		
		default:
			return FALSE;
	}
	
	return TRUE;
}

//==[ CODE ]============================================================
// LoadResourceDLLs
VOID LoadResourceDLLs(VOID)
{
	TCHAR szFileName[MAX_PATH] = { 0 };
	HANDLE hFile = NULL;
	WIN32_FIND_DATA wfd;
	HINSTANCE hModule = NULL;
	INIT_PROC lpInitProc = NULL;
	LPVOID lpvMemory = NULL;
	PTCHAR ptr = NULL;
	
	// Get the directory path of the application.
	// NOTE: Without the trailing backslash!
	GetModuleFileName(GetModuleHandle(NULL), szFileName, COUNTOF(szFileName));
	ptr = _tcsrchr(szFileName, '\\');
	if (ptr != NULL)
		*ptr = 0;
	else return;
	
	// Load the DLLs in the resource directory.
	_sntprintf(szFileName, COUNTOF(szFileName),
		TEXT("%s\\%s\\*.dll"), szFileName, STR_RESDIR);
	hFile = FindFirstFile(szFileName, &wfd);
	
	if (hFile != INVALID_HANDLE_VALUE) {
		do {
			// Load the resource DLL.
			// Even if we're not able to load the current DLL, we should try
			// loading the others. Therefore the use of "continue".
			hModule = LoadLibrary(wfd.cFileName);
			if (hModule == NULL)
				continue;
			
			lpInitProc = (INIT_PROC) GetProcAddress(hModule, "Initialize");
			if (lpInitProc == NULL)
				continue;
			
			// Allocate memory for the loaded resource DLL
			// If our request for memory space fails, then there is no reason
			// to try loading the other resource DLLs -- the request for
			// memory will fail as well. Therefore the use of "break".
			lpvMemory = HeapReAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY,
				g_paResources, sizeof(PRESOURCE) * (g_nResourcesCount + 1));
				
			if (lpvMemory == NULL)
				break;
				
			g_paResources = lpvMemory;
			
			g_paResources[g_nResourcesCount] =
				HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(RESOURCE));
			
			if (g_paResources[g_nResourcesCount] == NULL)
				break;
			
			// Register the newly loaded DLL.
			// If the loaded DLL fails to initialize successfully, we should
			// free the memory allocated for it.
			g_paResources[g_nResourcesCount]->hInstance = hModule;
			if ((*lpInitProc)(g_paResources[g_nResourcesCount])) {
				g_nResourcesCount++;
			} else {
				FreeLibrary(hModule);
				HeapFree(GetProcessHeap(), 0, g_paResources[g_nResourcesCount]);
			}
		} while (FindNextFile(hFile, &wfd));
		
		FindClose(hFile);
	}
}

//==[ CODE ]============================================================
// SetControlsText
VOID SetControlsText(HWND hwnd)
{
	SIZE sz;
	int  ctls[] = { IDC_LBL_LANGUAGE, IDC_LBL_VOICE, IDC_RBT_AUTOMATIC,
		IDC_RBT_SCHEDULE, IDC_BTN_TELLTIME, IDC_BTN_SAVE, IDC_BTN_CLOSE,
		IDC_LBL_STARTTIME, IDC_LBL_INTERVAL, IDC_BTN_ADD,
		IDC_BTN_REMOVE };
	int i, j, k;
	
	// Main window text
	_sntprintf(g_szBuffer, COUNTOF(g_szBuffer), TEXT("%s - ") STR_APPTITLE,
		GETSTRING(IDS_PREFS));
	SetWindowText(hwnd, g_szBuffer);
	
	// Core window controls.
	for (i = 0; i < COUNTOF(ctls); i++)
		SendDlgItemMessage(hwnd, ctls[i],
			WM_SETTEXT, 0, (LPARAM) GETSTRING(ctls[i]));

	ctls[0] = IDC_RBT_AUTOMATIC;
	ctls[1] = IDC_RBT_SCHEDULE;
	
	for (i = 0; i < 2; i++) {
		GETSTRING(ctls[i]);
		GetTextExtentPoint32(GetDC(GetDlgItem(hwnd, ctls[i])),
			g_szBuffer, _tcslen(g_szBuffer), &sz);
		SetWindowSize(GetDlgItem(hwnd, ctls[i]), sz.cx + 2, sz.cy);
	}
	
	ctls[0] = IDC_CBX_STARTTIME;
	ctls[1] = IDS_AM;
	ctls[2] = IDS_PM;
	ctls[3] = IDC_CBX_INTERVAL;
	ctls[4] = IDS_MINUTES;
	ctls[5] = IDS_HOURS;
	
	// i = combobox controls counter
	// j = combobox items counter
	// k = combobox current selection holder
	for (i = 0; i < 6; i += 3) {
		k = SendDlgItemMessage(hwnd, ctls[i], CB_GETCURSEL, 0, 0);
		SendDlgItemMessage(hwnd, ctls[i], CB_RESETCONTENT, 0, 0);
		
		for (j = 1; j < 3; j++) {
			SendDlgItemMessage(hwnd, ctls[i],
				CB_ADDSTRING, 0, (LPARAM) GETSTRING(ctls[i+j]));
		}
		
		if (k == CB_ERR) k = 0;
		SendDlgItemMessage(hwnd, ctls[i], CB_SETCURSEL, k, 0);
	}
	
	// Voices
	SendDlgItemMessage(hwnd, IDC_CBX_VOICE, CB_RESETCONTENT, 0, 0);
	for (i = 0; i < g_pActiveResource->nVoicesCount; i++)
		SendDlgItemMessage(hwnd, IDC_CBX_VOICE,
			CB_ADDSTRING, 0, (LPARAM) g_pActiveResource->paVoices[i]);
	i = SendDlgItemMessage(hwnd, IDC_CBX_VOICE,
		CB_FINDSTRINGEXACT, -1, (LPARAM) g_pActiveResource->pzActiveVoice);
	if (i == CB_ERR) i = 0;
	SendDlgItemMessage(hwnd, IDC_CBX_VOICE, CB_SETCURSEL, i, 0);
	SendMessage(hwnd, WM_COMMAND,
		MAKEWPARAM(IDC_CBX_VOICE, CBN_SELCHANGE),
			(LPARAM) GetDlgItem(hwnd, IDC_CBX_VOICE));
}

//==[ CODE ]============================================================
// ScheduleTime
VOID ScheduleTime(HWND hwnd, DWORD dwTime, BOOL bSort)
{
	DWORD dwData;
	int i;
	
	i = HIWORD(dwTime);
	_sntprintf(g_szBuffer, COUNTOF(g_szBuffer), TEXT("%d:%02d %s"),
		i % 12 ? i % 12 : 12, LOWORD(dwTime),
			i < 12 ? TEXT("AM") : TEXT("PM"));
	
	if (bSort) {
		i = SendDlgItemMessage(hwnd, IDC_LBX_SCHEDULE, LB_GETCOUNT, 0, 0);
		while (--i >= 0) {
			dwData = SendDlgItemMessage(hwnd,
				IDC_LBX_SCHEDULE, LB_GETITEMDATA, i, 0);
			if (dwData < dwTime)
				break;
		}
		SendDlgItemMessage(hwnd, IDC_LBX_SCHEDULE,
			LB_INSERTSTRING, i + 1, (LPARAM) g_szBuffer);
	} else {
		SendDlgItemMessage(hwnd, IDC_LBX_SCHEDULE, LB_ADDSTRING,
			0, (LPARAM) g_szBuffer);
	}
		
	i = SendDlgItemMessage(hwnd, IDC_LBX_SCHEDULE,
		LB_FINDSTRINGEXACT, -1, (LPARAM) g_szBuffer);
	if (i != LB_ERR)
		SendDlgItemMessage(hwnd, IDC_LBX_SCHEDULE,
			LB_SETITEMDATA, i, dwTime);
}

//==[ CODE ]============================================================
// LoadSettings
BOOL LoadSettings(VOID)
{
	HSQLITESTMT hSQLiteStmt;
	PTCHAR ptr;
	int i;
	
	i = SQLitePrepare(g_hSQLite, TEXT("SELECT * FROM [Setting]"),
		sizeof(TCHAR) * 23, &hSQLiteStmt, NULL);
	if (i != SQLITE_OK) {
		SQLiteFinalize(hSQLiteStmt);
		return FALSE;
	}
	
	while (SQLiteStep(hSQLiteStmt) == SQLITE_ROW) {
		switch (SQLiteColumnInt(hSQLiteStmt, 0)) {
			case IDQ_LANGUAGE:
				ptr = (PTCHAR) SQLiteColumnText(hSQLiteStmt, 1);
				if (ptr != NULL) {
					for (i = 0; i < g_nResourcesCount; i++) {
						if (_tcscmp(g_paResources[i]->pzLanguage, ptr) == 0) {
							g_pActiveResource = g_paResources[i];
							break;
						}
					}
				}
			break;
			
			case IDQ_VOICE:
				ptr = (PTCHAR) SQLiteColumnText(hSQLiteStmt, 1);
				if (ptr != NULL && g_pActiveResource != NULL) {
					for (i = 0; i < g_pActiveResource->nVoicesCount; i++) {
						if (!_tcscmp(g_pActiveResource->paVoices[i], ptr)) {
							g_pActiveResource->pzActiveVoice =
								g_pActiveResource->paVoices[i];
							break;
						}
					}
				}
				break;
			
			case IDQ_APPDATA:
				g_dwAppData = SQLiteColumnInt(hSQLiteStmt, 1);
				break;
			
			case IDQ_APPFLAGS:
				g_dwAppFlags = SQLiteColumnInt(hSQLiteStmt, 1);
				break;
		}
	}
	
	if (SQLiteFinalize(hSQLiteStmt) != SQLITE_OK)
		return FALSE;
	
	return TRUE;
}

//==[ CODE ]============================================================
// SaveSettings
BOOL SaveSettings(VOID)
{
	HSQLITESTMT hSQLiteStmt = NULL;
	BOOL bReturn = FALSE;
	
	if (SQLitePrepare(g_hSQLite,
		TEXT("UPDATE [Setting] SET [Data] = ? WHERE ID = ?"),
			sizeof(TCHAR) * 45, &hSQLiteStmt, NULL) != SQLITE_OK)
	{ goto end; }
	
	SQLiteBindInt(hSQLiteStmt, 2, IDQ_LANGUAGE);
	SQLiteBindText(hSQLiteStmt, 1, g_pActiveResource->pzLanguage, -1, SQLITE_STATIC);
	if (SQLiteStep(hSQLiteStmt) != SQLITE_DONE)
		goto end;
	
	// Selected Voice.
	if (SQLiteReset(hSQLiteStmt) != SQLITE_OK)
		goto end;
	SQLiteBindInt(hSQLiteStmt, 2, IDQ_VOICE);
	SQLiteBindText(hSQLiteStmt, 1, g_pActiveResource->pzActiveVoice, -1, SQLITE_STATIC);
	if (SQLiteStep(hSQLiteStmt) != SQLITE_DONE)
		goto end;
	
	// Application data.
	if (SQLiteReset(hSQLiteStmt) != SQLITE_OK)
		goto end;
	SQLiteBindInt(hSQLiteStmt, 2, IDQ_APPDATA);
	SQLiteBindInt(hSQLiteStmt, 1, g_dwAppData);
	if (SQLiteStep(hSQLiteStmt) != SQLITE_DONE)
		goto end;
	
	// Application flags.
	if (SQLiteReset(hSQLiteStmt) != SQLITE_OK)
		goto end;
	SQLiteBindInt(hSQLiteStmt, 2, IDQ_APPFLAGS);
	SQLiteBindInt(hSQLiteStmt, 1, g_dwAppFlags);
	if (SQLiteStep(hSQLiteStmt) != SQLITE_DONE)
		goto end;
	
	bReturn = TRUE;
	
	end:
	SQLiteFinalize(hSQLiteStmt);
	return bReturn;
}

//==[ CODE ]============================================================
// SchedNextTimer
VOID SchedNextTimer(HWND hwnd)
{
	// Wait time is in seconds!
	DWORD dwWaitTime = 0;
	DWORD dwCurrentTime = 0;
	DWORD dwSchedTime = 0;
	SYSTEMTIME st;
	
	GetLocalTime(&st);
	
	if (g_dwAppFlags & APP_AUTOMATIC) {
		DWORD dwStartTime;
		DWORD dwInterval;
		
		// Get the start time in minutes.
		dwStartTime =
			LOBYTE(LOWORD(g_dwAppData)) * 60 + HIBYTE(LOWORD(g_dwAppData));
		
		// Get play time interval in minutes.
		dwInterval = LOBYTE(HIWORD(g_dwAppData));
		if (dwInterval > 59)
			dwInterval = (dwInterval - 59) * 60;
		
		// Convert current time into minutes.
		dwCurrentTime = st.wHour * 60 + st.wMinute;
		
		// Determine the next time to run.
		if (dwCurrentTime < dwStartTime)
			dwSchedTime = dwStartTime;
		
		while (dwSchedTime <= dwCurrentTime)
			dwSchedTime += dwInterval;
		
		dwWaitTime = (dwSchedTime - dwCurrentTime) * 60 - st.wSecond;
	}
	
	// SPOILER ALERT!!
	else {
		int i, count;
		
		dwCurrentTime = MAKELONG(st.wMinute, st.wHour);
		count = SendDlgItemMessage(hwnd, IDC_LBX_SCHEDULE, LB_GETCOUNT, 0, 0);
		
		if (count > 0) {
			for (i = 0; i < count; i++) {
				dwSchedTime = SendDlgItemMessage(hwnd,
					IDC_LBX_SCHEDULE, LB_GETITEMDATA, i, 0);
				if (dwSchedTime > dwCurrentTime)
					break;
			}
			
			// We couldn't get any next schedule to run, so we default
			// to first schedule on the list.
			// NOTE: We're looking for the NEXT time to run so any schedule
			// same as the current time should not be processed.
			if (dwSchedTime <= dwCurrentTime) {
				dwSchedTime = SendDlgItemMessage(hwnd,
					IDC_LBX_SCHEDULE, LB_GETITEMDATA, 0, 0);
			}
			
			// Convert the scheduled and current time to minutes.
			dwSchedTime = HIWORD(dwSchedTime) * 60 + LOWORD(dwSchedTime);
			dwCurrentTime = st.wHour * 60 + st.wMinute;
			
			// If the scheduled time is less than the current time,
			// the we should wrap around the 00:00 hour mark i.e. 1,440 mins.
			if (dwSchedTime < dwCurrentTime)
				dwSchedTime += 1440;
			
			dwWaitTime = (dwSchedTime - dwCurrentTime) * 60 - st.wSecond;
		}
	}
	
	// Schedule new timer.
	KillTimer(hwnd, IDT_TELLTIME);
	if (dwWaitTime > 0) {
		SetTimer(hwnd, IDT_TELLTIME, dwWaitTime * 1000, NULL);
	}
}

//==[ CODE ]============================================================
// PlayTime
DWORD WINAPI PlayTime(LPVOID lpVoid)
{
	WORD waves[5];
	SYSTEMTIME st;
	int i;
	
	GetLocalTime(&st);
	
	waves[0] = IDR_WAV_DING;
	waves[1] = IDR_WAV_V1INTRO;
	waves[2] = IDR_WAV_V1MINUTE0 + (st.wHour % 12 ? st.wHour % 12 : 12);
	waves[3] = IDR_WAV_V1MINUTE0 + st.wMinute;
	waves[4] = st.wHour < 12 ? IDR_WAV_V1AM : IDR_WAV_V1PM;
	
	for (i = 0; i < COUNTOF(waves); i++) {
		PlaySound(MAKEINTRESOURCE(waves[i]),
			GetModuleHandle(NULL), SND_RESOURCE | SND_SYNC);
	}
	
	return 0;
}
